import torch
import torchvision
import torchvision.transforms as transforms
from torch.utils.data import DataLoader

from nets.ghostnet import ghostnet

transform_test = transforms.Compose([
    transforms.CenterCrop(960),
    transforms.Resize(512),
    transforms.ToTensor()
])

testset = torchvision.datasets.ImageFolder(root='./dataset/test', transform=transform_test)
testloader = DataLoader(testset, batch_size=8, shuffle=False, num_workers=8)

# 使用gpu
use_cuda = torch.cuda.is_available()
device = torch.device("cuda" if use_cuda else "cpu")

# 模型定义
net = ghostnet().to(device)


def test():
    print("Load checkpoint...")
    net.load_state_dict(torch.load('./model_MESSIDOR/model_MESSIDOR_3000.pkl'))

    print("test begin！")
    with torch.no_grad():
        accuracy = 0
        total = 0
        for data in testloader:
            # 开始测试
            net.eval()

            images, labels = data
            images, labels = images.to(device), labels.to(device)

            outputs = net(images)

            _, predicted = torch.max(outputs.data, 1)  # 返回每一行中最大值的那个元素，且返回其索引(得分高的那一类)
            total += labels.size(0)

            accuracy += (predicted == labels).sum()
        accuracy = accuracy.item()  # acc为tensor, total为int型, acc/total自动将小数点后数字归零. 所以需要先将acc通过.item()转化成int型.

        # 输出测试准确率
        print('测试准确率为: {:.2f}%'.format(100 * accuracy / total))


if __name__ == '__main__':
    test()