import torch
import torchvision
import torch.nn as nn
import torch.optim as optim
from torch.utils.data import DataLoader
import torchvision.transforms as transforms
import os
import sys

from nets.ghostnet import ghostnet

sys.path.append(os.getcwd())
os.environ["CUDA_VISIBLE_DEVICES"] = "0"  # 指定一块gpu为可见

# #############创建数据加载器###################
print('data loaded begin!')
# 预处理
data_transform = transforms.Compose([
    transforms.CenterCrop(960),
    transforms.Resize(512),
    transforms.ToTensor()
])
# 加载训练集数据
train_dataset = torchvision.datasets.ImageFolder(root='./datasets/train', transform=data_transform)
train_data = DataLoader(train_dataset, batch_size=8, shuffle=True, num_workers=8, drop_last=True)
# 加载验证集数据
val_dataset = torchvision.datasets.ImageFolder(root='./datasets/val', transform=data_transform)
val_data = DataLoader(val_dataset, batch_size=8, shuffle=True, num_workers=8, drop_last=True)

print(type(train_data))
print('data loaded done!')

# ##################创建网络模型###################
model = ghostnet()

# ##############训练#################
device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
if torch.cuda.device_count() > 1:
    model = nn.DataParallel(model)
model = model.to(device)
criterion = nn.CrossEntropyLoss()
optimizer = optim.Adam(model.parameters(), weight_decay=0.001)

nums_epoch = 3001  # 训练epoch的数量，动态调整

print('training begin!')
# 开始训练
train_losses = []
train_acces = []
val_losses = []
val_acces = []
S = []

for epoch in range(nums_epoch):
    train_loss = 0
    train_acc = 0
    model = model.train()
    # 训练开始
    print('Epoch ' + str(epoch + 1) + ' begin!')
    for img_train, label_train in train_data:
        img_train = img_train.to(device)
        label_train = label_train.to(device)

        # 前向传播
        out = model(img_train)
        optimizer.zero_grad()
        loss = criterion(out, label_train)
        # 反向传播
        loss.backward()

        optimizer.step()

        # 记录误差
        train_loss += loss.item()
        # 计算分类的准确率
        _, pred = out.max(1)
        num_correct = (pred == label_train).sum().item()
        acc = num_correct / img_train.shape[0]
        train_acc += acc
    train_losses.append(train_loss / len(train_data))
    train_acces.append(train_acc / len(train_data))
    print('Epoch' + str(epoch + 1) + ' Train  done!')

    val_loss = 0
    val_acc = 0
    # 验证开始
    print('Epoch' + str(epoch + 1) + ' Val  begin!')
    for img_val, label_val in val_data:
        img_val = img_val.to(device)
        label_val = label_val.to(device)

        out = model(img_val)
        loss = criterion(out, label_val)

        # 记录误差
        val_loss += loss.item()
        # 计算分类的准确率
        _, pred = out.max(1)
        num_correct = (pred == label_val).sum().item()
        acc = num_correct / img_val.shape[0]
        val_acc += acc
    val_losses.append(val_loss / len(val_data))
    val_acces.append(val_acc / len(val_data))
    print('Epoch' + str(epoch + 1) + ' Val  done!')

    print('Epoch {} ,Train Loss: {} ,Train  Accuracy: {} ,Test Loss: {} ,Test Accuracy: {}'.format(
        epoch + 1, train_loss / len(train_data), train_acc / len(train_data), val_loss / len(val_data),
        val_acc / len(val_data)))

    s = ('Epoch {} ,Train Loss: {} ,Train  Accuracy: {} ,Test Loss: {} ,Test Accuracy: {}'.format(
        epoch + 1, train_loss / len(train_data), train_acc / len(train_data), val_loss / len(val_data),
        val_acc / len(val_data)))

    S.append(s)

    if epoch % 100 == 0:
        torch.save(model.state_dict(), './models/model_MESSIDOR/model_MESSIDOR_{}.pkl'.format(epoch))

    print('model saved done!')

